@@ -10,7 +10,7 @@ from typing import (
     Iterable,
     List,
     Mapping,
-    Tuple,
+    NamedTuple,
     TypeVar,
     Union,
 )
@@ -28,25 +28,11 @@ from di.api.scopes import Scope
 from di.exceptions import IncompatibleDependencyError
 
 
-class ExecutionState:
-    __slots__ = (
-        "stacks",
-        "results",
-        "values",
-        "cache",
-    )
-
-    def __init__(
-        self,
-        stacks: Mapping[Scope, Union[AsyncExitStack, ExitStack]],
-        results: List[Any],
-        cache: ScopeMap[CacheKey, Any],
-        values: Mapping[DependencyProvider, Any],
-    ):
-        self.stacks = stacks
-        self.results = results
-        self.cache = cache
-        self.values = values
+class ExecutionState(NamedTuple):
+    stacks: Mapping[Scope, Union[AsyncExitStack, ExitStack]]
+    results: List[Any]
+    cache: ScopeMap[CacheKey, Any]
+    values: Mapping[DependencyProvider, Any]
 
 
 DependencyType = TypeVar("DependencyType")
@@ -68,7 +54,8 @@ class Task:
     )
 
     compute: Union[
-        Callable[[ExecutionState], None], Callable[[ExecutionState], Awaitable[None]]
+        Callable[[ExecutionState], Awaitable[None]],
+        Callable[[ExecutionState], None],
     ]
     wrapped_call: DependencyProvider
     user_function: DependencyProvider
@@ -82,7 +69,7 @@ class Task:
         dependant: DependantBase[Any],
         task_id: int,
         positional_parameters: Iterable[Task],
-        keyword_parameters: Iterable[Tuple[str, Task]],
+        keyword_parameters: Mapping[str, Task],
     ) -> None:
         self.scope = scope
         self.user_function = call
@@ -115,17 +102,18 @@ class Task:
                 self.compute = self.compute_sync_func_no_cache
 
         self.call_user_func_with_deps = self.generate_execute_fn(
-            positional_parameters, keyword_parameters
+            self.wrapped_call, positional_parameters, keyword_parameters
         )
 
     def __hash__(self) -> int:
         return self.task_id
 
+    @staticmethod
     def generate_execute_fn(
-        self,
+        call: DependencyProvider,
         positional_parameters: Iterable[Task],
-        keyword_parameters: Iterable[Tuple[str, Task]],
-    ) -> Callable[[Callable[..., Any], List[Any]], Any]:
+        keyword_parameters: Mapping[str, Task],
+    ) -> Callable[[List[Any]], Any]:
         # this codegen speeds up argument collection and passing
         # by avoiding creation of intermediary containers to store the values
         positional_arg_template = "results[{}]"
@@ -133,11 +121,12 @@ class Task:
         args: List[str] = []
         for task in positional_parameters:
             args.append(positional_arg_template.format(task.task_id))
-        for keyword, task in keyword_parameters:
+        for keyword, task in keyword_parameters.items():
             args.append(keyword_arg_template.format(keyword, task.task_id))
-        out: Dict[str, Callable[..., Any]] = {}
-        exec(f'def execute(call, results): return call({",".join(args)})', out)
-        return out["execute"]
+        locals: Dict[str, Any] = {}
+        globals = {"call": call}
+        exec(f'def execute(results): return call({",".join(args)})', globals, locals)
+        return locals["execute"]  # type: ignore[no-any-return]
 
     def __repr__(self) -> str:
         return (
@@ -152,9 +141,7 @@ class Task:
         if value is not UNSET:
             state.results[self.task_id] = value
             return
-        dependency_value = await self.call_user_func_with_deps(
-            self.wrapped_call, state.results
-        )
+        dependency_value = await self.call_user_func_with_deps(state.results)
         state.results[self.task_id] = dependency_value
         state.cache.set(self.cache_key, dependency_value, scope=self.scope)
 
@@ -162,9 +149,7 @@ class Task:
         if self.user_function in state.values:
             state.results[self.task_id] = state.values[self.user_function]
             return
-        dependency_value = await self.call_user_func_with_deps(
-            self.wrapped_call, state.results
-        )
+        dependency_value = await self.call_user_func_with_deps(state.results)
         state.results[self.task_id] = dependency_value
 
     async def compute_async_cm_cache(self, state: ExecutionState) -> None:
@@ -180,11 +165,11 @@ class Task:
         except AttributeError:
             raise IncompatibleDependencyError(
                 f"The dependency {self.user_function} is an awaitable dependency"
-                f" and canot be used in the sync scope {self.scope}"
+                f" and cannot be used in the sync scope {self.scope}"
             ) from None
 
         dependency_value: Any = await enter(
-            self.call_user_func_with_deps(self.wrapped_call, state.results)
+            self.call_user_func_with_deps(state.results)
         )
         state.results[self.task_id] = dependency_value
         state.cache.set(self.cache_key, dependency_value, scope=self.scope)
@@ -198,10 +183,10 @@ class Task:
         except AttributeError:
             raise IncompatibleDependencyError(
                 f"The dependency {self.user_function} is an awaitable dependency"
-                f" and canot be used in the sync scope {self.scope}"
+                f" and cannot be used in the sync scope {self.scope}"
             ) from None
         dependency_value: Any = await enter(
-            self.call_user_func_with_deps(self.wrapped_call, state.results)
+            self.call_user_func_with_deps(state.results)
         )
         state.results[self.task_id] = dependency_value
 
@@ -214,7 +199,7 @@ class Task:
             state.results[self.task_id] = value
             return
         val = state.stacks[self.scope].enter_context(
-            self.call_user_func_with_deps(self.wrapped_call, state.results)
+            self.call_user_func_with_deps(state.results)
         )
         state.results[self.task_id] = val
         state.cache.set(self.cache_key, val, scope=self.scope)
@@ -224,7 +209,7 @@ class Task:
             state.results[self.task_id] = state.values[self.user_function]
             return
         state.results[self.task_id] = state.stacks[self.scope].enter_context(
-            self.call_user_func_with_deps(self.wrapped_call, state.results)
+            self.call_user_func_with_deps(state.results)
         )
 
     def compute_sync_func_cache(self, state: ExecutionState) -> None:
@@ -235,7 +220,7 @@ class Task:
         if value is not UNSET:
             state.results[self.task_id] = value
             return
-        val = self.call_user_func_with_deps(self.wrapped_call, state.results)
+        val = self.call_user_func_with_deps(state.results)
         state.results[self.task_id] = val
         state.cache.set(self.cache_key, val, scope=self.scope)
 
@@ -244,6 +229,4 @@ class Task:
             state.results[self.task_id] = state.values[self.user_function]
             return
 
-        state.results[self.task_id] = self.call_user_func_with_deps(
-            self.wrapped_call, state.results
-        )
+        state.results[self.task_id] = self.call_user_func_with_deps(state.results)
