@@ -1,7 +1,7 @@
 import sys
 from dataclasses import dataclass
 from types import TracebackType
-from typing import Any, Generic, Optional, Type, TypeVar, Union
+from typing import Generic, Optional, Type, TypeVar, Union
 
 if sys.version_info < (3, 8):  # pragma: no cover
     from typing_extensions import Protocol
@@ -47,5 +47,5 @@ class FusedContextManager(Generic[T]):
 
 
 @dataclass
-class Some:
-    value: Any
+class Some(Generic[T]):
+    value: T
