@@ -1,7 +1,5 @@
-from __future__ import annotations
-
 import typing
-from typing import Any, Generic, List, Mapping, TypeVar
+from typing import Any, Generic, Iterable, Mapping, TypeVar
 
 from di.api.dependencies import DependantBase, DependencyParameter
 
@@ -16,7 +14,7 @@ class SolvedDependant(Generic[DependencyType]):
     __slots__ = ("dependency", "dag", "container_cache")
 
     dependency: DependantBase[DependencyType]
-    dag: Mapping[DependantBase[Any], List[DependencyParameter]]
+    dag: Mapping[DependantBase[Any], Iterable[DependencyParameter]]
     # container_cache can be used by the creating container to store data that is tied
     # to the SolvedDependant
     container_cache: typing.Any
@@ -24,15 +22,9 @@ class SolvedDependant(Generic[DependencyType]):
     def __init__(
         self,
         dependency: DependantBase[DependencyType],
-        dag: Mapping[DependantBase[Any], List[DependencyParameter]],
+        dag: Mapping[DependantBase[Any], Iterable[DependencyParameter]],
         container_cache: typing.Any,
     ):
         self.dependency = dependency
         self.dag = dag
         self.container_cache = container_cache
-
-    def get_flat_subdependants(self) -> List[DependantBase[Any]]:
-        """Get an exhaustive list of all of the dependencies of this dependency,
-        in no particular order.
-        """
-        return list(self.dag.keys() - {self.dependency})
